__version__ = '0.1.2'

from joker.clients.monolog import MonologInterface
from joker.clients.printable import PDFClient
from joker.clients.files import FileStorageInterface

if __name__ == '__main__':
    print(__version__)



#!/usr/bin/env python3
# coding: utf-8

import logging
import mimetypes
import os.path
import smtplib
from email.message import EmailMessage
from typing import Union, List

_logger = logging.getLogger(__name__)


class ExtendedEmailMessage(EmailMessage):
    @staticmethod
    def _fmt_addrs(addrs: Union[List[str], str], sep=', '):
        if isinstance(addrs, str):
            return addrs
        return sep.join(addrs)

    def set_subject_and_addrs(
            self, subject: str, from_addr: str,
            to_addrs: Union[List[str], str],
            cc_addrs: Union[List[str], str] = None):
        self['Subject'] = subject
        self['From'] = from_addr
        self['To'] = self._fmt_addrs(to_addrs)
        if cc_addrs:
            self['Cc'] = self._fmt_addrs(cc_addrs)

    def add_attachment_from_local_file(self, path: str):
        mime = mimetypes.guess_type(path)[0]
        maintype, subtype = mime.split('/', 1)
        filename = os.path.split(path)[1]
        with open(path, 'rb') as fin:
            content = fin.read()
            self.add_attachment(
                content, filename=filename,
                maintype=maintype, subtype=subtype,
            )


class EmailInterface:
    def __init__(
            self, host: str, port: int,
            username: str, password: str,
            from_addr: str, fake=False):
        """
        Args:
            host:
            port:
            username:
            password:
            from_addr:
            fake: if true, messages will NOT be send actually
        """
        self.host = host
        self.port = port
        self.username = username
        self.password = password
        self.from_addr = from_addr
        self.fake = fake

    def _get_smtp(self):
        smtp = smtplib.SMTP(self.host, self.port)
        smtp.login(self.username, self.password)
        return smtp

    def _send(self, from_addr, to_addrs, msg):
        smtp = self._get_smtp()
        try:
            resp = smtp.sendmail(from_addr, to_addrs, msg)
            _logger.info("smtp.sendmail() => %s", resp)
        finally:
            smtp.quit()
            _logger.info("smtp.quit()")
        return resp

    def send(
            self,
            subject: str,
            content: str,
            attachments: list,
            from_addr: Union[str, None],
            to_addrs: Union[List[str], str],
            cc_addrs: Union[List[str], str] = None,
            content_subtype='html'):
        """
        Args:
            subject:
            content:
            attachments
            from_addr: override default from_addr
            to_addrs:
            cc_addrs:
            content_subtype:
        """
        from_addr = from_addr or self.from_addr
        mail = ExtendedEmailMessage()
        mail.set_subject_and_addrs(
            subject, from_addr, to_addrs, cc_addrs,
        )
        mail.set_content(content, subtype=content_subtype)
        for att in attachments:
            mail.add_attachment_from_local_file(att)

        msg = mail.as_string()
        _logger.debug('EmailMessage().as_string()[:100] => %s', msg[:100])
        _logger.info('EmailMessage().fake => %s', self.fake)
        if not self.fake:
            return self._send(from_addr, to_addrs, msg)



#!/usr/bin/env python3
# coding: utf-8
from __future__ import annotations

import os
import typing
import urllib.parse
import zipfile
from dataclasses import dataclass
from functools import cached_property

import requests

PathLike = typing.Union[str, os.PathLike]


class MemberFile(typing.TypedDict):
    cid: str
    filename: str


@dataclass
class FileStorageInterface:
    base_url: str
    credential: dict

    @cached_property
    def session(self):
        sess = requests.session()
        url = urllib.parse.urljoin(self.base_url, '/login')
        sess.post(url, data=self.credential)
        return sess

    def save(self, content: bytes) -> str:
        url = urllib.parse.urljoin(self.base_url, '/files')
        resp = self.session.post(url, files={'file': content})
        return resp.json()['data']

    def load(self, cid: str) -> bytes:
        url = urllib.parse.urljoin(self.base_url, f'/files/{cid}')
        resp = requests.get(url)
        return resp.content

    def create_archive(self, path: PathLike, memberfiles: list[MemberFile]):
        with zipfile.ZipFile(path, "w") as zipf:
            for m in memberfiles:
                content = self.load(m['cid'])
                with zipf.open(m['filename'], 'w') as fout:
                    fout.write(content)



#!/usr/bin/env python3
# coding: utf-8
from __future__ import annotations

import json
import logging
import urllib.parse

import requests

from joker.clients.utils import _BaseHTTPClient

_logger = logging.getLogger(__name__)


class PDFClient(_BaseHTTPClient):
    def _generate(self, tpl_path: str, data: dict) -> (bytes, str):
        url = urllib.parse.urljoin(self.base_url, tpl_path)
        headers = {'Content-type': 'application/json'}
        payload = json.dumps(data, default=str)
        _logger.info('initial url: %r', url)
        resp = requests.post(url, data=payload, headers=headers)
        _logger.info('redirected url: %r', resp.url)
        _logger.info(
            'content: %s bytes, %r',
            len(resp.content), resp.content[:100],
        )
        return resp.content, resp.url

    def generate(self, tpl_path: str, data: dict) -> bytes:
        return self._generate(tpl_path, data)[0]



#!/usr/bin/env python3
# coding: utf-8
from __future__ import annotations

import json
import logging
import shlex
from functools import cached_property
from json import JSONDecodeError
from urllib.parse import urlparse

import requests
# noinspection PyUnresolvedReferences
from joker.meta.utils import *
from volkanic.errors import TechnicalError
from volkanic.introspect import razor

_logger = logging.getLogger(__name__)


class ResponseDict(dict):
    @property
    def code(self) -> int:
        return self.get('code', 3)

    @property
    def data(self):
        return self.get('data')

    @property
    def message(self):
        return self.get('message', 'OK')


def dump_json_request_to_curl(method: str, url: str, data=None, aslist=False):
    method = method.upper()
    if method == 'GET':
        parts = ['curl', url]
    else:
        parts = [
            'curl', '-X', method, url,
            '-H', 'Content-Type: application/json',
            '-d', json.dumps(razor(data), ensure_ascii=False),
        ]
    if aslist:
        return parts
    parts = [shlex.quote(s) for s in parts]
    return ' '.join(parts)


def _log_bad_response(resp: requests.Response):
    _logger.error(
        'bad response: %s %r',
        resp.status_code, resp.content[:1000]
    )


def _decode_response(resp: requests.Response):
    status = resp.status_code
    if status >= 400:
        raise TechnicalError(f'got response status code {status}')
    try:
        rd = ResponseDict(resp.json())
    except JSONDecodeError:
        raise TechnicalError('cannot decode json')
    if rd.code != 0:
        raise TechnicalError(f'error response ({rd.code})')
    if rd.message:
        _logger.info(rd.message)
    return rd.data


def decode_response(resp: requests.Response):
    try:
        return _decode_response(resp)
    except TechnicalError:
        _log_bad_response(resp)
        raise


class _BaseHTTPClient:
    @staticmethod
    def _check_url(url: str):
        path = urlparse(url).path
        if path == '' or path.endswith('/'):
            return
        raise ValueError('service url path must end with "/"')

    def __init__(self, url: str):
        self._check_url(url)
        self.base_url = url
        c = self.__class__.__name__
        _logger.info('new %s instance, %r', c, url)

    @cached_property
    def session(self):
        return requests.session()
